//設定用
const Config = {
  //画面の解像度
  Screen: {
    Width: 256,//幅
    Height: 256,//高さ
    BackGroundColor: 0x444444,//背景色
  },
  Keys: { //キーボード入力
    Up: "w",
    Right: "d",
    Down: "s",
    Left: "a",
    A: "n",
    B: "m",
    Start: "Enter"
  },
}

//読みこむデータリスト(名前：ファイルパス)
const Resource = {
  Player: 'assets/player.png',
}

//上のリストを読み込みやすい形に変えてます
const assets = [];
for (let key in Resource) {
  assets.push(Resource[key]);
}

let core;//ゲームの基幹プログラム用の変数
let inputManager;

//ブラウザの読み込みが完了したら実行されます
window.onload = () => {
  //設定した画面サイズでcoreを作成
  core = new Game(Config.Screen.Width, Config.Screen.Height, Config.Screen.BackGroundColor);
  core.app.view.id = "game-screen";//cssのidを設定
  inputManager = new InputManager();

  //データのロード
  core.preload(assets);

  //読み込み完了でメインシーンに切り替わります
  core.onload = () => {
    core.replaceScene(new MainScene());
  }
}

let player;
let bullet;
//メインシーン
class MainScene extends Container {
  constructor(){
    super();
    
    player = new EnchantSprite(16, 16);
    this.addChild(player);
    player.image = core.resources[Resource.Player].texture; 
    player.position.set(Config.Screen.Width*0.5, Config.Screen.Height*0.5);
    player.anchor.set(0.5, 0.5)

    bullet = new Graphics();
    this.addChild(bullet);
    bullet.circFill(0, 0, 4, 0xff0000);
    bullet.y = -10;
  }
  update(delta){
    super.update(delta);
    
    const oblique = 1 / Math.sqrt(2);//ななめ移動の値
    switch(inputManager.checkDirection()){
      case inputManager.keyDirections.UP:
        player.y--;
        break;
      case inputManager.keyDirections.UP_RIGHT:
        player.x += oblique;
        player.y -= oblique;
        break;
      case inputManager.keyDirections.RIGHT:
        player.x++;
        break;
      case inputManager.keyDirections.DOWN_RIGHT:
        player.x += oblique;
        player.y += oblique;
        break;
      case inputManager.keyDirections.DOWN:
        player.y++;
        break;
      case inputManager.keyDirections.DOWN_LEFT:
        player.x -= oblique;
        player.y += oblique;
        break;
      case inputManager.keyDirections.LEFT:
        player.x--;
        break;
      case inputManager.keyDirections.UP_LEFT:
        player.x -= oblique;
        player.y -= oblique;
        break;
      default:
        break;
    }

    if(inputManager.checkButton("A") == inputManager.keyStatus.DOWN){
      if(bullet.y <= -20){
        bullet.clear();
        bullet.circFill(0, 0, 4, 0xff0000);
        bullet.y = player.y;
        bullet.x = player.x;
      }
    }
    if(inputManager.checkButton("B") == inputManager.keyStatus.DOWN){
      if(bullet.y <= -20){
        bullet.clear();
        bullet.circFill(0, 0, 8, 0x00ffff);
        bullet.y = player.y;
        bullet.x = player.x;
      }
    }
    if(inputManager.checkButton("Start") == inputManager.keyStatus.DOWN){
      core.pushScene(new PauseScene());
    }   

    bullet.y -= 4;
  }
}

class PauseScene extends Container {
  constructor(){
    super();
    
    const t = new PIXI.Text('PAUSE', new PIXI.TextStyle({
      fontFamily: 'sans-serif',
      fontSize: 32,
      fill: 0xffffff,
    }));
    t.anchor.set(0.5, 0.5);
    t.position.set(Config.Screen.Width*0.5, Config.Screen.Height*0.5);
    this.addChild(t);
  }
  update(delta){
    super.update(delta);
    if(inputManager.checkButton("Start") == inputManager.keyStatus.DOWN){
      core.popScene();
    }  
  }
}

class InputManager {
  constructor() {
    //方向入力チェック用定数
    this.keyDirections = {
      UP: 1,
      UP_RIGHT: 3,
      RIGHT: 2,
      DOWN_RIGHT: 6,
      DOWN: 4,
      DOWN_LEFT: 12,
      LEFT: 8,
      UP_LEFT: 9,
    };
    //キーの状態管理定数
    this.keyStatus = {
      HOLD: 2,
      DOWN: 1,
      UNDOWN: 0,
      RELEASE: -1,
    };
    //キーの状態管理用変数
    this.input = {
      //入力されたキーのチェック用
      keys: {
        Up: false,
        Right: false,
        Down: false,
        Left: false,
        A: false,
        B: false,
        Start: false
      },
      //一つ前のキーの状態管理用
      keysPrev: {
        Up: false,
        Right: false,
        Down: false,
        Left: false,
        A: false,
        B: false,
        Start: false
      },
    };

    //スマホ・タブレットの時だけv-pad表示
    if (navigator.userAgent.match(/iPhone|iPad|Android/)) {
      this.vpad = new Vpad(this.input);
    }

    //キーを押した時
    document.addEventListener('keydown', (e) => {
      switch(e.key){
        case Config.Keys.Up:
          this.input.keys.Up = true;
          break;
        case Config.Keys.Down:
          this.input.keys.Down = true;
          break;
        case Config.Keys.Right:
          this.input.keys.Right = true;
          break;
        case Config.Keys.Left:
          this.input.keys.Left = true;
          break;
        case Config.Keys.A:
          this.input.keys.A = true;
          break;
        case Config.Keys.B:
          this.input.keys.B = true;
          break;
        case Config.Keys.Start:
          this.input.keys.Start = true;
          break;
      }
    });

    //キーを離したとき
    document.addEventListener('keyup', (e) => {
      switch(e.key){
        case Config.Keys.Up:
          this.input.keys.Up = false;
          break;
        case Config.Keys.Down:
          this.input.keys.Down = false;
          break;
        case Config.Keys.Right:
          this.input.keys.Right = false;
          break;
        case Config.Keys.Left:
          this.input.keys.Left = false;
          break;
        case Config.Keys.A:
          this.input.keys.A = false;
          break;
        case Config.Keys.B:
          this.input.keys.B = false;
          break;
        case Config.Keys.Start:
          this.input.keys.Start = false;
          break;
      }
    });
  }

  //方向キー入力チェック
  checkDirection() {
    let direction = 0;//初期化
    if(this.input.keys.Up){
        direction += this.keyDirections.UP;
    }
    if(this.input.keys.Right){
      direction += this.keyDirections.RIGHT;
    }
    if(this.input.keys.Down){
      direction += this.keyDirections.DOWN;
    }
    if(this.input.keys.Left){
      direction += this.keyDirections.LEFT;
    }
    return direction;
  }

  //ボタンの入力状態をチェックして返す
  checkButton(key) {
    if(this.input.keys[key]){
      if(this.input.keysPrev[key] == false){
        this.input.keysPrev[key] = true;
        return this.keyStatus.DOWN;//押されたとき
      }
      return this.keyStatus.HOLD;//押しっぱなし
    }else{
      if(this.input.keysPrev[key] == true){
        this.input.keysPrev[key] = false;
        return this.keyStatus.RELEASE;//ボタンを離した時
      }
      return this.keyStatus.UNDOWN;//押されていない
    }
  }
}

/******************************************************
 * バーチャルパッド
 ******************************************************/
class Vpad {
  constructor(input){
    this.input = input;//InputManagerのinput
    this.resizePad();
    // リサイズイベントの登録
    window.addEventListener('resize', ()=>{this.resizePad();});
  }
  //画面サイズが変わるたびにvpadも作り変える
  resizePad(){
    let styleDisplay = "block";//ゲームパッド対策
    //すでにあれば一度削除する
    if(this.pad != undefined){
      styleDisplay = this.pad.style.display;//ゲームパッド対策
      while(this.pad.firstChild){
        this.pad.removeChild(this.pad.firstChild);
      }
      this.pad.parentNode.removeChild(this.pad);
    }

    const screen = document.getElementById("game-screen");//ゲーム画面

    //HTMLのdivでvpad作成
    const pad = document.createElement('div');
    document.body.appendChild(pad);
    this.pad = pad;
    pad.id = "pad";
    pad.style.width = screen.style.width;
    pad.style.display = styleDisplay;
    
    //タッチで拡大とか起こるのを防ぐ
    pad.addEventListener("touchstart", (e) => {
      e.preventDefault();
    });
    pad.addEventListener("touchmove", (e) => {
      e.preventDefault();
    });

    //横長の場合位置変更
    if(window.innerWidth > window.innerHeight){
      pad.style.width = `${window.innerWidth}px`;
      pad.style.position = "absolute";//画面の上にかぶせるため
      pad.style.backgroundColor = "transparent";//透明
      pad.style.bottom = "0";//下に固定
    }
    const height = Number(screen.style.height.split('px')[0]) * 0.5;//ゲーム画面の半分の高さをゲームパッドの高さに
    pad.style.height = `${height}px`;
    
    //方向キー作成
    new DirKey(this.pad, this.input, height);
    
    //Aボタン作成
    let style = {
      width: `${height * 0.5}px`,
      height: `${height * 0.5}px`,
      right: `${height * 0.5}px`,
      top: `${height * 0.4}px`,
      borderRadius: "50%"
    }
    new ActBtn(this.pad, this.input, "A", "A", style);

    //Bボタン作成
    style = {
      width: `${height * 0.5}px`,
      height: `${height * 0.5}px`,
      right: `${height * 0.05}px`,
      top: `${height * 0.1}px`,
      borderRadius: "50%"
    }
    new ActBtn(this.pad, this.input, "B", "B", style);

    //STARTボタン作成
    style = {
      width: `${height * 0.3}px`,
      height: `${height * 0.15}px`,
      right: `${height * 0.65}px`,
      top: `${height * 0.05}px`,
      borderRadius: `${height * 0.15 * 0.5}px`
    }
    new ActBtn(this.pad, this.input, "Start", "START", style);
  }
}

//方向キークラス
class DirKey {
   constructor(parent, input, padHeight) {
    this.isTouching = false;
    this.originX = 0;
    this.originY = 0;

    //HTMLのdivでキーのエリアを作成
    const div = document.createElement('div');
    parent.appendChild(div);
    div.className = "dir-key";
    div.style.width = div.style.height = `${padHeight * 0.8}px`;
    div.style.left = `${padHeight * 0.05}px`;
    div.style.top = `${padHeight * 0.1}px`;
    this.maxRadius = padHeight * 0.15;//中心移動させる半径
    this.emptySpace = padHeight * 0.05;//あそび

    //十字キーのボタン(張りぼて。タッチイベントはない)
    const up = document.createElement('div');
    up.className = "dir up";
    div.appendChild(up);
    const left = document.createElement('div');
    left.className = "dir left";
    div.appendChild(left);
    const right = document.createElement('div');
    right.className = "dir right";
    div.appendChild(right);
    const down = document.createElement('div');
    down.className = "dir down";
    div.appendChild(down);
    const mid = document.createElement('div');
    mid.className = "dir mid";
    div.appendChild(mid);
    const circle = document.createElement('div');
    circle.className = "circle";
    mid.appendChild(circle);
    
    //タッチイベント
    div.addEventListener("touchstart", (e) => {
      e.preventDefault();
      this.isTouching = true;    
      //タッチした位置を原点にする
      this.originX = e.targetTouches[0].clientX;
      this.originY = e.targetTouches[0].clientY;
    });

    div.addEventListener("touchmove", (e) => {
      e.preventDefault();
      if(!this.isTouching) return;
      dirReset();//からなず一度リセット
      
      //タッチ位置を取得
      const posX = e.targetTouches[0].clientX;
      const posY = e.targetTouches[0].clientY;

      //原点からの移動量を計算
      let vecY = posY - this.originY;
      let vecX = posX - this.originX;
      let vec = Math.sqrt(vecX * vecX + vecY * vecY);
      if(vec < this.emptySpace)return;//移動が少ない時は反応しない(遊び)

      const rad = Math.atan2(posY - this.originY, posX - this.originX);
      const y = Math.sin(rad);
      const x = Math.cos(rad);

      //移動幅が大きいときは中心を移動させる
      if(vec > this.maxRadius){
        this.originX = posX - x * this.maxRadius;
        this.originY = posY - y * this.maxRadius;
      }
     
      const abs_x = Math.abs(x);
      const abs_y = Math.abs(y);
      if(abs_x > abs_y){//xの方が大きい場合左右移動となる
        if(x < 0){//マイナスであれば左
          input.keys.Left = true;
        }else{
          input.keys.Right = true;
        }
        if(abs_x <= abs_y * 2){//2yがxより大きい場合斜め入力と判断
          if(y < 0){//マイナスであれば上
            input.keys.Up = true;
          }else{
            input.keys.Down = true;
          }
        }
      }else{//yの方が大きい場合上下移動となる
        if(y < 0){//マイナスであれば上
          input.keys.Up = true;
        }else{
          input.keys.Down = true;
        }
        if(abs_y <= abs_x * 2){//2xがyより大きい場合斜め入力と判断
          if(x < 0){//マイナスであれば左
            input.keys.Left = true;
          }else{
            input.keys.Right = true;
          }
        }
      }    
    });
    
    div.addEventListener("touchend", (e) => {
      dirReset();
    });

    const dirReset = () => {
      input.keys.Right = input.keys.Left = input.keys.Up = input.keys.Down = false;
    }
  }
}
//アクションボタンクラス
class ActBtn {
  constructor(parent, input, key, name, style) {
    //HTMLのdivでボタンを作成
    const div = document.createElement('div');
    div.className = "button";
    parent.appendChild(div);
    div.style.width = style.width;
    div.style.height = style.height;
    div.style.right = style.right;
    div.style.top = style.top;
    div.style.borderRadius = style.borderRadius;

    //ボタン名を表示
    const p = document.createElement('p');
    p.innerHTML = name;
    div.appendChild(p);

    //タッチスタート
    div.addEventListener("touchstart", (e) => {
      e.preventDefault();
      input.keys[key] = true;
    });
    
    //タッチエンド
    div.addEventListener("touchend", (e) => {
      input.keys[key] = false;
    });
  }
}
